@@ -359,7 +359,7 @@ class FeatureHistogram {
           continue;
         }
 
-        // mark to is splittable
+        // mark as able to be split
         is_splittable_ = true;
         // better split point
         if (current_gain > best_gain) {
@@ -572,7 +572,7 @@ class FeatureHistogram {
     const double cnt_factor = num_data / sum_hessian;
     // from right to left, and we don't need data in bin0
     for (; t >= t_end; --t) {
-      if (static_cast<uint32_t>(t + offset) < threshold) {
+      if (static_cast<uint32_t>(t + offset) <= threshold) {
         break;
       }
 
@@ -940,20 +940,24 @@ class FeatureHistogram {
           continue;
         }
 
-        // mark to is splittable
+        // mark as able to be split
         is_splittable_ = true;
         // better split point
         if (current_gain > best_gain) {
+          if (USE_MC) {
+            best_right_constraints = constraints->RightToBasicConstraint();
+            best_left_constraints = constraints->LeftToBasicConstraint();
+            if (best_right_constraints.min > best_right_constraints.max ||
+                best_left_constraints.min > best_left_constraints.max) {
+              continue;
+            }
+          }
           best_left_count = left_count;
           best_sum_left_gradient = sum_left_gradient;
           best_sum_left_hessian = sum_left_hessian;
           // left is <= threshold, right is > threshold.  so this is t-1
           best_threshold = static_cast<uint32_t>(t - 1 + offset);
           best_gain = current_gain;
-          if (USE_MC) {
-            best_right_constraints = constraints->RightToBasicConstraint();
-            best_left_constraints = constraints->LeftToBasicConstraint();
-          }
         }
       }
     } else {
@@ -1006,7 +1010,7 @@ class FeatureHistogram {
         }
 
         double sum_right_hessian = sum_hessian - sum_left_hessian;
-        // if sum hessian too small
+        // if sum Hessian too small
         if (sum_right_hessian < meta_->config->min_sum_hessian_in_leaf) {
           break;
         }
@@ -1029,19 +1033,23 @@ class FeatureHistogram {
           continue;
         }
 
-        // mark to is splittable
+        // mark as able to be split
         is_splittable_ = true;
         // better split point
         if (current_gain > best_gain) {
+          if (USE_MC) {
+            best_right_constraints = constraints->RightToBasicConstraint();
+            best_left_constraints = constraints->LeftToBasicConstraint();
+            if (best_right_constraints.min > best_right_constraints.max ||
+                best_left_constraints.min > best_left_constraints.max) {
+              continue;
+            }
+          }
           best_left_count = left_count;
           best_sum_left_gradient = sum_left_gradient;
           best_sum_left_hessian = sum_left_hessian;
           best_threshold = static_cast<uint32_t>(t + offset);
           best_gain = current_gain;
-          if (USE_MC) {
-            best_right_constraints = constraints->RightToBasicConstraint();
-            best_left_constraints = constraints->LeftToBasicConstraint();
-          }
         }
       }
     }
