@@ -55,8 +55,8 @@ void VotingParallelTreeLearner<TREELEARNER_T>::Init(const Dataset* train_data, b
   larger_buffer_read_start_pos_.resize(this->num_features_);
   global_data_count_in_leaf_.resize(this->config_->num_leaves);
 
-  smaller_leaf_splits_global_.reset(new LeafSplits(train_data->num_data()));
-  larger_leaf_splits_global_.reset(new LeafSplits(train_data->num_data()));
+  smaller_leaf_splits_global_.reset(new LeafSplits(train_data->num_data(), this->config_));
+  larger_leaf_splits_global_.reset(new LeafSplits(train_data->num_data(), this->config_));
 
   local_config_ = *this->config_;
   local_config_.min_data_in_leaf /= num_machines_;
@@ -67,9 +67,8 @@ void VotingParallelTreeLearner<TREELEARNER_T>::Init(const Dataset* train_data, b
   // initialize histograms for global
   smaller_leaf_histogram_array_global_.reset(new FeatureHistogram[this->num_features_]);
   larger_leaf_histogram_array_global_.reset(new FeatureHistogram[this->num_features_]);
-  std::vector<int> offsets;
-  int num_total_bin = HistogramPool::GetNumTotalHistogramBins(
-      train_data, this->share_state_->is_colwise, &offsets);
+  std::vector<uint32_t> offsets = this->share_state_->feature_hist_offsets();
+  int num_total_bin = this->share_state_->num_hist_total_bin();
   smaller_leaf_histogram_data_.resize(num_total_bin * 2);
   larger_leaf_histogram_data_.resize(num_total_bin * 2);
   HistogramPool::SetFeatureInfo<true, true>(train_data, this->config_, &feature_metas_);
@@ -260,9 +259,52 @@ void VotingParallelTreeLearner<TREELEARNER_T>::FindBestSplits(const Tree* tree)
   }
   TREELEARNER_T::ConstructHistograms(is_feature_used, use_subtract);
 
+  const int smaller_leaf_index = this->smaller_leaf_splits_->leaf_index();
+  const data_size_t local_data_on_smaller_leaf = this->data_partition_->leaf_count(smaller_leaf_index);
+  if (local_data_on_smaller_leaf <= 0) {
+    // clear histogram buffer before synchronizing
+    // otherwise histogram contents from the previous iteration will be sent
+    OMP_INIT_EX();
+    #pragma omp parallel for schedule(static)
+    for (int feature_index = 0; feature_index < this->num_features_; ++feature_index) {
+      OMP_LOOP_EX_BEGIN();
+      if (!is_feature_used[feature_index]) { continue; }
+      const BinMapper* feature_bin_mapper = this->train_data_->FeatureBinMapper(feature_index);
+      const int num_bin = feature_bin_mapper->num_bin();
+      const int offset = static_cast<int>(feature_bin_mapper->GetMostFreqBin() == 0);
+      hist_t* hist_ptr = this->smaller_leaf_histogram_array_[feature_index].RawData();
+      std::memset(reinterpret_cast<void*>(hist_ptr), 0, (num_bin - offset) * kHistEntrySize);
+      OMP_LOOP_EX_END();
+    }
+    OMP_THROW_EX();
+  }
+
+  if (this->larger_leaf_splits_ != nullptr) {
+    const int larger_leaf_index = this->larger_leaf_splits_->leaf_index();
+    if (larger_leaf_index >= 0) {
+      const data_size_t local_data_on_larger_leaf = this->data_partition_->leaf_count(larger_leaf_index);
+      if (local_data_on_larger_leaf <= 0) {
+        OMP_INIT_EX();
+        #pragma omp parallel for schedule(static)
+        for (int feature_index = 0; feature_index < this->num_features_; ++feature_index) {
+          OMP_LOOP_EX_BEGIN();
+          if (!is_feature_used[feature_index]) { continue; }
+          const BinMapper* feature_bin_mapper = this->train_data_->FeatureBinMapper(feature_index);
+          const int num_bin = feature_bin_mapper->num_bin();
+          const int offset = static_cast<int>(feature_bin_mapper->GetMostFreqBin() == 0);
+          hist_t* hist_ptr = this->larger_leaf_histogram_array_[feature_index].RawData();
+          std::memset(reinterpret_cast<void*>(hist_ptr), 0, (num_bin - offset) * kHistEntrySize);
+          OMP_LOOP_EX_END();
+        }
+        OMP_THROW_EX();
+      }
+    }
+  }
+
   std::vector<SplitInfo> smaller_bestsplit_per_features(this->num_features_);
   std::vector<SplitInfo> larger_bestsplit_per_features(this->num_features_);
-
+  double smaller_leaf_parent_output = this->GetParentOutput(tree, this->smaller_leaf_splits_.get());
+  double larger_leaf_parent_output = this->GetParentOutput(tree, this->larger_leaf_splits_.get());
   OMP_INIT_EX();
   // find splits
 #pragma omp parallel for schedule(static)
@@ -278,7 +320,8 @@ void VotingParallelTreeLearner<TREELEARNER_T>::FindBestSplits(const Tree* tree)
         this->smaller_leaf_histogram_array_, feature_index, real_feature_index,
         true, this->smaller_leaf_splits_->num_data_in_leaf(),
         this->smaller_leaf_splits_.get(),
-        &smaller_bestsplit_per_features[feature_index]);
+        &smaller_bestsplit_per_features[feature_index],
+        smaller_leaf_parent_output);
     // only has root leaf
     if (this->larger_leaf_splits_ == nullptr || this->larger_leaf_splits_->leaf_index() < 0) { continue; }
 
@@ -292,7 +335,8 @@ void VotingParallelTreeLearner<TREELEARNER_T>::FindBestSplits(const Tree* tree)
         this->larger_leaf_histogram_array_, feature_index, real_feature_index,
         true, this->larger_leaf_splits_->num_data_in_leaf(),
         this->larger_leaf_splits_.get(),
-        &larger_bestsplit_per_features[feature_index]);
+        &larger_bestsplit_per_features[feature_index],
+        larger_leaf_parent_output);
     OMP_LOOP_EX_END();
   }
   OMP_THROW_EX();
@@ -354,8 +398,9 @@ void VotingParallelTreeLearner<TREELEARNER_T>::FindBestSplitsFromHistograms(cons
       this->col_sampler_.GetByNode(tree, this->smaller_leaf_splits_->leaf_index());
   std::vector<int8_t> larger_node_used_features =
       this->col_sampler_.GetByNode(tree, this->larger_leaf_splits_->leaf_index());
+  double smaller_leaf_parent_output = this->GetParentOutput(tree, this->smaller_leaf_splits_global_.get());
+  double larger_leaf_parent_output = this->GetParentOutput(tree, this->larger_leaf_splits_global_.get());
   // find best split from local aggregated histograms
-
   OMP_INIT_EX();
 #pragma omp parallel for schedule(static) num_threads(this->share_state_->num_threads)
   for (int feature_index = 0; feature_index < this->num_features_; ++feature_index) {
@@ -375,7 +420,8 @@ void VotingParallelTreeLearner<TREELEARNER_T>::FindBestSplitsFromHistograms(cons
           smaller_leaf_histogram_array_global_.get(), feature_index,
           real_feature_index, smaller_node_used_features[feature_index],
           GetGlobalDataCountInLeaf(smaller_leaf_splits_global_->leaf_index()),
-          smaller_leaf_splits_global_.get(), &smaller_bests_per_thread[tid]);
+          smaller_leaf_splits_global_.get(), &smaller_bests_per_thread[tid],
+          smaller_leaf_parent_output);
     }
 
     if (larger_is_feature_aggregated_[feature_index]) {
@@ -391,7 +437,8 @@ void VotingParallelTreeLearner<TREELEARNER_T>::FindBestSplitsFromHistograms(cons
           real_feature_index,
           larger_node_used_features[feature_index],
           GetGlobalDataCountInLeaf(larger_leaf_splits_global_->leaf_index()),
-          larger_leaf_splits_global_.get(), &larger_bests_per_thread[tid]);
+          larger_leaf_splits_global_.get(), &larger_bests_per_thread[tid],
+          larger_leaf_parent_output);
     }
     OMP_LOOP_EX_END();
   }
@@ -454,6 +501,7 @@ void VotingParallelTreeLearner<TREELEARNER_T>::Split(Tree* tree, int best_Leaf,
 }
 
 // instantiate template classes, otherwise linker cannot find the code
+template class VotingParallelTreeLearner<CUDATreeLearner>;
 template class VotingParallelTreeLearner<GPUTreeLearner>;
 template class VotingParallelTreeLearner<SerialTreeLearner>;
 }  // namespace LightGBM
